@@ -47,7 +47,15 @@ class Tide(object):
 
     def __init__(self, constituents = None, amplitudes = None, phases = None,
                  model = None, radians = False):
-        """Initialise a tidal model. """
+        """
+        Initialise a tidal model. Provide constituents, amplitudes and phases OR a model.
+        Arguments:
+        constituents -- list of constituents used in the model.
+        amplitudes -- list of amplitudes corresponding to constituents
+        phases -- list of phases corresponding to constituents
+        model -- an ndarray of type Tide.dtype representing the constituents, amplitudes and phases.
+        radians -- boolean representing whether phases are in radians (default False)
+        """
         if None not in [constituents, amplitudes, phases]:
             if len(constituents) == len(amplitudes) == len(phases):
                 model = np.zeros(len(phases), dtype=Tide.dtype)
@@ -67,10 +75,19 @@ class Tide(object):
         self.normalize()
     
     def prepare(self, *args, **kwargs):
+
         return Tide._prepare(self.model['constituent'], *args, **kwargs)
 
     @staticmethod 
     def _prepare(constituents, t0, t = None, radians = True):
+        """
+        Return constituent speed and equilibrium argument at a given time, and constituent node factors at given times.
+        Arguments:
+        constituents -- list of constituents to prepare
+        t0 -- time at which to evaluate speed and equilibrium argument for each constituent
+        t -- list of times at which to evaluate node factors for each constituent (default: t0)
+        radians -- whether to return the angular arguments in radians or degrees (default: True)
+        """
         #The equilibrium argument is constant and taken at the beginning of the time series (t0).
         #The speed of the equilibrium argument changes very slowly, so again we take it to be constant
         #over any length of data. The node factors change more rapidly.
@@ -98,29 +115,51 @@ class Tide(object):
         return speed, u, f, V0
          
     def at(self, t):
-        if isinstance(t, Iterable):
-            #We'll do this a bit more sensibly soon
-            return np.array([self.at(ti)[0] for ti in t])
-        speed,[u],[f],V0 = self.prepare(t, radians=True)
-        t = np.zeros(1)
-        amplitudes = self.model['amplitude'][:, np.newaxis]
-        phases = d2r*self.model['phase'][:, np.newaxis]
-        return Tide._tidal_series(t, amplitudes, phases, speed,u,f,V0)
+        """
+        Return the modelled tidal height at given times.
+        Arguments:
+        t -- array of times at which to evaluate the tidal height
+        """
+        t0 = t[0]
+        hours = self._hours(t0, t)
+        partition = 240.0
+        t = self._partition(hours, partition)
+        times = self._times(t0, [(i + 0.5)*partition for i in range(len(t))])
+        speed, u, f, V0 = self.prepare(t0, times, radians = True)
+        H = self.model['amplitude'][:, np.newaxis]
+        p = d2r*self.model['phase'][:, np.newaxis]
+        return np.concatenate([Tide._tidal_series(t_i, H, p, speed, u_i, f_i, V0) for t_i, u_i, f_i in izip(t, u, f)])
     
-    def highs(self, args):
+    def highs(self, *args):
+        """
+        Generator yielding only the high tides.
+        Arguments:
+        see Tide.extrema()
+        """
         for t in ifilter(lambda e: e[2] == 'H', self.extrema(*args)):
             yield t
             
-    def lows(self, args):
-        for t in ifilter(lambda e: e[2] == 'H', self.extrema(*args)):
+    def lows(self, *args):
+        """
+        Generator yielding only the low tides.
+        Arguments:
+        see Tide.extrema()
+        """
+        for t in ifilter(lambda e: e[2] == 'L', self.extrema(*args)):
             yield t
     
     def form_number(self):
+        """
+        Returns the model's form number, a helpful heuristic for classifying tides.
+        """
         k1, o1, m2, s2 = (np.extract(self.model['constituent'] == c, self.model['amplitude']) for c
                           in [constituent._K1, constituent._O1, constituent._M2, constituent._S2])
         return (k1+o1)/(m2+s2)
     
     def classify(self):
+        """
+        Classify the tide according to its form number
+        """
         form = self.form_number()
         if 0 <= form <= 0.25:
             return 'semidiurnal'
@@ -132,20 +171,29 @@ class Tide(object):
             return 'diurnal'
         
     def extrema(self, t0, t1 = None, partition = 2400.0):
+        """
+        A generator for high and low tides.
+        Arguments:
+        t0 -- time after which extrema are sought
+        t1 -- optional time before which extrema are sought (if not given, the generator is infinite)
+        partition -- number of hours for which we consider the node factors to be constant (default: 2400.0)
+        """
         if t1:
             #yield from in python 3.4
             for e in takewhile(lambda t: t[0] < t1, self.extrema(t0)):
                 yield e
         else:
+            #We assume that extrema are separated by at least delta hours
             delta = np.amin([90.0 / c.speed(astro(t0)) for c in self.model['constituent'] if not c.speed(astro(t0)) == 0])
             #We search for stationary points from offset hours before t0 to ensure we find any which
             #might occur very soon after t0.
             offset = 24.0
-            amplitude = self.model['amplitude'][:, np.newaxis]
-            phase = d2r*self.model['phase'][:, np.newaxis]
             partitions = (Tide._times(t0, i*partition) for i in count()), (Tide._times(t0, i*partition) for i in count(1))
             #We'll overestimate to be on the safe side; values outside (start,end) won't get yielded.
             interval_count = int(np.ceil((partition + offset) / delta)) + 1
+            amplitude = self.model['amplitude'][:, np.newaxis]
+            phase = d2r*self.model['phase'][:, np.newaxis]
+
             for start, end in izip(*partitions):
                 speed, [u], [f], V0 = self.prepare(start, Tide._times(start, 0.5*partition))
                 # These derivatives don't include the time dependence of u or f, but these change slowly.
@@ -159,13 +207,19 @@ class Tide(object):
                     if d(a)*d(b) < 0:
                         extrema = fsolve(d, (a + b) / 2.0, fprime = d2)[0]
                         time = Tide._times(start, extrema)
-                        height = self.at(time)
+                        [height] = self.at([time])
                         type = 'H' if d2(extrema) < 0 else 'L'
                         if start < time < end:
                             yield (time, height, type)
     
     @staticmethod
     def _hours(t0, t):
+        """
+        Return the hourly offset(s) of a (list of) time from a given time.
+        Arguments:
+        t0 -- time from which offsets are sought
+        t -- times to find hourly offsets from t0.
+        """
         if not isinstance(t, Iterable):
             return Tide._hours(t0, [t])[0]
         elif isinstance(t[0], datetime):
@@ -175,6 +229,12 @@ class Tide(object):
         
     @staticmethod
     def _partition(hours, partition = 3600.0):
+        """
+        Partition a sorted list of numbers (or in this case hours).
+        Arguments:
+        hours -- sorted ndarray of hours.
+        partition -- maximum partition length (default: 3600.0)
+        """
         partition = float(partition)
         relative = hours - hours[0]
         total_partitions = np.ceil(relative[-1] / partition + 10*np.finfo(np.float).eps).astype('int')
@@ -182,20 +242,27 @@ class Tide(object):
     
     @staticmethod
     def _times(t0, hours):
+        """
+        Return a (list of) datetime(s) given an initial time and an (list of) hourly offset(s).
+        Arguments:
+        t0 -- initial time
+        hours -- hourly offsets from t0
+        """
         if not isinstance(hours, Iterable):
             return Tide._times(t0, [hours])[0]
         elif not isinstance(hours[0], datetime):
             return np.array([t0 + timedelta(hours=h) for h in hours])
         else:
             return np.array(hours)
-    ##Return the modelled heights for given times
-    ##t is of shape (nt,); all other arguments are of shape (nc, 1)
+        
     @staticmethod
     def _tidal_series(t, amplitude, phase, speed, u, f, V0):     
         return np.sum(amplitude*f*np.cos(speed*t + (V0 + u) - phase), axis=0)
     
-    #By convention amplitudes, phases are positive
     def normalize(self):
+        """
+        Adapt self.model so that amplitudes are positive and phases are in [0,360) as per convention
+        """
         for i, (_, amplitude, phase) in enumerate(self.model):
             if amplitude < 0:
                 self.model['amplitude'][i] = -amplitude
@@ -204,6 +271,20 @@ class Tide(object):
 
     @classmethod
     def decompose(cls, heights, t = None, t0 = None, interval=None, constituents=constituent.noaa, initial = None, n_period = 2, callback = None, full_output = False):
+        """
+        Return an instance of Tide which has been fitted to a series of tidal observations.
+        Arguments:
+        It is not necessary to provide t0 or interval if t is provided.
+        heights -- ndarray of tidal observation heights
+        t -- ndarray of tidal observation times
+        t0 -- datetime representing the time at which heights[0] was recorded
+        interval -- hourly interval between readings
+        constituents -- list of constituents to use in the fit (default: constituent.noaa)
+        initial -- optional Tide instance to use as first guess for least squares solver
+        n_period -- only include constituents which complete at least this many periods (default: 2)
+        callback -- optional function to be called at each iteration of the solver
+        full_output -- whether to return the output of scipy's leastsq solver (default: False)
+        """
         if t is not None:
             if isinstance(t[0], datetime):
                 hours = Tide._hours(t[0], t)
@@ -221,7 +302,7 @@ class Tide(object):
         constituents = list(OrderedDict.fromkeys(constituents))
 
         #No need for least squares to find the mean water level constituent z0, work relative to mean
-        constituents = [c for c in constituents if not c == constituent._Zo]
+        constituents = [c for c in constituents if not c == constituent._Z0]
         z0 = np.mean(heights)
         heights = heights - z0
 
@@ -277,7 +358,7 @@ class Tide(object):
         lsq = leastsq(residual, initial, Dfun=D_residual, col_deriv=True, ftol=1e-7) # 
 
         model = np.zeros(1+n, dtype=cls.dtype)
-        model[0] = (constituent._Zo, z0, 0)
+        model[0] = (constituent._Z0, z0, 0)
         model[1:]['constituent'] = constituents[:]
         model[1:]['amplitude'] = lsq[0][:n]
         model[1:]['phase'] = lsq[0][n:]
